package cn.yundev.xzy.biz.impl;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import net.sourceforge.cardme.util.Util;
import net.sourceforge.cardme.vcard.EncodingType;
import net.sourceforge.cardme.vcard.VCard;
import net.sourceforge.cardme.vcard.VCardImpl;
import net.sourceforge.cardme.vcard.VCardVersion;
import net.sourceforge.cardme.vcard.features.AddressFeature;
import net.sourceforge.cardme.vcard.features.PhotoFeature;
import net.sourceforge.cardme.vcard.features.TelephoneFeature;
import net.sourceforge.cardme.vcard.types.AddressType;
import net.sourceforge.cardme.vcard.types.FormattedNameType;
import net.sourceforge.cardme.vcard.types.NameType;
import net.sourceforge.cardme.vcard.types.PhotoType;
import net.sourceforge.cardme.vcard.types.TelephoneType;
import net.sourceforge.cardme.vcard.types.VersionType;
import net.sourceforge.cardme.vcard.types.media.ImageMediaType;
import net.sourceforge.cardme.vcard.types.parameters.AddressParameterType;
import net.sourceforge.cardme.vcard.types.parameters.ParameterTypeStyle;
import net.sourceforge.cardme.vcard.types.parameters.TelephoneParameterType;
import net.sourceforge.cardme.vcard.types.parameters.XAddressParameterType;
import cn.yundev.xzy.biz.service.VcardService;
import cn.yundev.xzy.dataobject.AddressBookDO;

public class VcardServiceImpl implements VcardService {

	@Override
	public VCardImpl getVcard(AddressBookDO book) {
		VCardImpl vcard = new VCardImpl();
		vcard.setVersion(new VersionType(VCardVersion.V3_0));
		NameType name = new NameType();
		// name.setFamilyName("Doe");
		name.setGivenName(book.getName());
		if (book.getSex().equals("女")) {
			name.addHonorificPrefix("女士");
		} else {
			name.addHonorificPrefix("先生");
		}
		vcard.setName(name);
		vcard.setFormattedName(new FormattedNameType(book.getName()));

		AddressFeature address1 = new AddressType();

		address1.setStreetAddress(book.getAddress());
		address1.addAddressParameterType(AddressParameterType.HOME);
		address1.addAddressParameterType(AddressParameterType.PARCEL);
		address1.addAddressParameterType(AddressParameterType.PREF);
		address1.addExtendedAddressParameterType(new XAddressParameterType(
				"CUSTOM-PARAM-TYPE"));
		address1.addExtendedAddressParameterType(new XAddressParameterType(
				"CUSTOM-PARAM-TYPE", "WITH-CUSTOM-VALUE"));
		vcard.addAddress(address1);

		if (book.getPhone() != null) {
			TelephoneFeature telephone = new TelephoneType();
			telephone.setTelephone(book.getPhone());
			telephone.addTelephoneParameterType(TelephoneParameterType.CELL);
			telephone
					.setParameterTypeStyle(ParameterTypeStyle.PARAMETER_VALUE_LIST);
			vcard.addTelephoneNumber(telephone);
		}
		if (book.getTel() != null) {
			TelephoneFeature telephone1 = new TelephoneType();
			telephone1.setTelephone(book.getTel());
			telephone1.addTelephoneParameterType(TelephoneParameterType.HOME);
			telephone1
					.setParameterTypeStyle(ParameterTypeStyle.PARAMETER_VALUE_LIST);
			vcard.addTelephoneNumber(telephone1);
		}
		if (book.getFox() != null) {
			TelephoneFeature telephone3 = new TelephoneType();
			telephone3.setTelephone(book.getFox());
			telephone3.addTelephoneParameterType(TelephoneParameterType.FAX);
			telephone3
					.setParameterTypeStyle(ParameterTypeStyle.PARAMETER_VALUE_LIST);
			vcard.addTelephoneNumber(telephone3);
		}
		
		String avatarString = book.getAvatar();
		if (avatarString != null) {
			PhotoFeature photo = new PhotoType();
			photo.setCompression(false);
			photo.setEncodingType(EncodingType.BINARY);
			photo.setImageMediaType(ImageMediaType.PNG);
			byte[] tuxPicture;
			try {
				tuxPicture = Util.getFileAsBytes(new File("src/main/webapp/"
						+ avatarString));
				photo.setPhoto(tuxPicture);
				vcard.addPhoto(photo);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		return vcard;
	}

	@Override
	public AddressBookDO getBook(VCard vcard) {
		AddressBookDO book = new AddressBookDO();
		book.setName(vcard.getName().getFamilyName()
				+ vcard.getName().getGivenName());
		book.setSex("男");
		Iterator<TelephoneFeature> tel = vcard.getTelephoneNumbers();
		while (tel.hasNext()) {
			TelephoneFeature t = tel.next();
			if (t.getTelephone() != "") {
				book.setPhone(t.getTelephone());
				break;
			}
		}
		Iterator<AddressFeature> address = vcard.getAddresses();
		while (address.hasNext()) {
			AddressFeature a = address.next();
			book.setAddress(a.getStreetAddress());
		}
		return book;
	}

}
