package cn.yundev.xzy.controller;

import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.DigestUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import cn.yundev.xzy.biz.service.UserService;
import cn.yundev.xzy.dataobject.ReturnJsonDO;
import cn.yundev.xzy.dataobject.UserDO;
import cn.yundev.xzy.dataobject.UserLoginDO;

import com.google.gson.Gson;


@Controller
@RequestMapping(value = {"","/"})
public class LoginController {
	
	@Autowired
	private HttpServletRequest request;
	
	@Autowired
	private UserService userService;
	
	/**
	 * Simply selects the home view to render by returning its name.
	 */
	@RequestMapping(value = {"","/","index.htm","login.htm","index"}, method = RequestMethod.GET)
	public String login(Locale locale, Model model) {
		request.getRequestURI();
		return "login";
	}
	
	@RequestMapping(value = {"/reg.htm"}, method = RequestMethod.GET)
	public String reg(Model model) {
		return "reg";
	}
	
	@RequestMapping(value = "dologin.htm", method = RequestMethod.POST)
	public void doLogin(Model model,Writer writer, @Valid UserLoginDO UserLoginDO,BindingResult BindingResult) {
		ReturnJsonDO result = new ReturnJsonDO();
		if (BindingResult.hasErrors()) {
			List<ObjectError> errors = BindingResult.getAllErrors();
			String msgString = "";
			for(ObjectError value:errors) {
				msgString += value.getDefaultMessage() + "\r\n";
				//break;
			}
			result.setStatus(0);
			result.setMsg(msgString);
			showJson(result, writer);
			return;
		}
		String username = UserLoginDO.getUsername();
		String password = UserLoginDO.getPassword();
		
		password = DigestUtils.md5DigestAsHex(password.getBytes());
		UserDO user = userService.getUserByName(username);

		if (user != null && user.getPassword().equals(password)) {
			request.getSession().setAttribute("islogin", "yes");
			request.getSession().setAttribute("user", user);
			result.setStatus(1);
			HashMap<String, String> data = new HashMap<String, String>();
			data.put("url", "main.htm");
			result.setData(data);
		} else {
			result.setStatus(0);
			result.setMsg("用户名或密码不正确");
		}
		showJson(result, writer);
	}
	
	@RequestMapping(value = "doreg.htm", method = RequestMethod.POST)
	public void doReg(Writer writer, @Valid UserDO UserDO,BindingResult BindingResult) {
		ReturnJsonDO result = new ReturnJsonDO();
		if (BindingResult.hasErrors()) {
			List<ObjectError> errors = BindingResult.getAllErrors();
			String msgString = "";
			for(ObjectError value:errors) {
				msgString += value.getDefaultMessage() + "\r\n";
				//break;
			}
			result.setStatus(0);
			result.setMsg(msgString);
			showJson(result, writer);
			return;
		}
		UserDO.setPassword(DigestUtils.md5DigestAsHex(UserDO.getPassword().getBytes()));
		if (!userService.insert(UserDO)) {
			result.setStatus(0);
			result.setMsg("已存在的用户名");
			showJson(result, writer);
		}
		request.getSession().setAttribute("islogin", "yes");
		request.getSession().setAttribute("user", UserDO);
		HashMap<String, String> data = new HashMap<String, String>();
		data.put("url", "main.htm");
		result.setStatus(1);
		result.setData(data);
		showJson(result, writer);
	}
	
	private void showJson(ReturnJsonDO result, Writer writer) {
		Gson gson = new Gson();
		try {
			writer.write(gson.toJson(result));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
