package cn.yundev.xzy.controller;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Writer;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import net.sourceforge.cardme.engine.VCardEngine;
import net.sourceforge.cardme.io.BinaryFoldingScheme;
import net.sourceforge.cardme.io.CompatibilityMode;
import net.sourceforge.cardme.io.FoldingScheme;
import net.sourceforge.cardme.io.VCardWriter;
import net.sourceforge.cardme.util.VCardUtils;
import net.sourceforge.cardme.vcard.VCard;
import net.sourceforge.cardme.vcard.VCardImpl;
import net.sourceforge.cardme.vcard.VCardVersion;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.DigestUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import cn.yundev.xzy.biz.service.AddressBookService;
import cn.yundev.xzy.biz.service.VcardService;
import cn.yundev.xzy.common.interceptor.RequireLogin;
import cn.yundev.xzy.dataobject.AddressBookDO;
import cn.yundev.xzy.dataobject.ReturnJsonDO;
import cn.yundev.xzy.dataobject.UserDO;
import cn.yundev.xzy.util.PageUtility;

import com.google.gson.Gson;

@Controller
@RequireLogin
public class MainController {

	@Autowired
	private AddressBookService addressBookService;

	@Autowired
	private VcardService vcardService;

	@Autowired
	private HttpServletRequest request;

	@RequestMapping(value = "main.htm", method = RequestMethod.GET)
	public String login(Model model) {
		String pageStr = request.getParameter("page");
		int page = 1;
		if (pageStr != null && pageStr != "") {
			try {
				page = Integer.parseInt(pageStr);
			} catch (Exception e) {
			}
		}
		int perpage = 12;
		int start = (page - 1) * perpage;
		int uid = ((UserDO) request.getSession().getAttribute("user")).getUid();
		int total = addressBookService.getByUidCount(uid);
		List<AddressBookDO> data = addressBookService.getList(uid, start,
				perpage);
		model.addAttribute("page", PageUtility.page(total, page, perpage));
		model.addAttribute("data", data);
		return "main";
	}

	@RequestMapping(value = "add.htm", method = RequestMethod.GET)
	public String add(Model model) {
		return "add";
	}

	@RequestMapping(value = "edit.htm", method = RequestMethod.GET)
	public String edit(Model model) {
		String idStr = request.getParameter("id");
		int id = 0;
		int uid = ((UserDO) request.getSession().getAttribute("user")).getUid();
		if (idStr != null && idStr != "") {
			try {
				id = Integer.parseInt(idStr);
			} catch (Exception e) {
			}
		}
		if (id < 0)
			return "error";
		AddressBookDO book = addressBookService.getById(id);
		if (book != null && book.getUid() != uid) {
			return "error";
		}
		model.addAttribute("book", book);
		return "add";
	}

	@RequestMapping(value = "upload.htm", method = RequestMethod.GET)
	public String update(Model model) {
		return "upload";
	}

	@RequestMapping(value = "doupload.htm", method = RequestMethod.POST)
	public String doUpdate(Model model, HttpServletRequest request,
			@RequestParam("img") MultipartFile file) throws IOException {
		if (file.isEmpty()) {
			model.addAttribute("error", "请选择要上传的图片");
			return "doupload";
		}
		if (file.getSize() > 1024*600) {
			model.addAttribute("error", "图片不能大于600k");
			return "doupload";
		}
		String filename = file.getOriginalFilename();
		String extName = this.getExtensionName(filename);
		if (!extName.equals("png") && !extName.equals("jpg") && !extName.equals("gif")) {
			model.addAttribute("error", "只允许上传png、jpg、gif格式的图片");
			return "doupload";
		}
		
		String fileString = this.copyFile(file.getInputStream(),
				file.getOriginalFilename());
		model.addAttribute("filename", "/resources/upload/" + fileString);
		return "doupload";
	}

	@RequestMapping(value = "doadd.htm", method = RequestMethod.POST)
	public void doAdd(Writer writer, @Valid AddressBookDO AddressBookDO,
			BindingResult BindingResult) {
		ReturnJsonDO result = new ReturnJsonDO();
		if (BindingResult.hasErrors()) {
			List<ObjectError> errors = BindingResult.getAllErrors();
			String msgString = "";
			for (ObjectError value : errors) {
				msgString += value.getDefaultMessage() + "\r\n";
			}
			result.setStatus(0);
			result.setMsg(msgString);
			showJson(result, writer);
			return;
		}
		AddressBookDO.setUid(((UserDO) request.getSession()
				.getAttribute("user")).getUid());
		addressBookService.insert(AddressBookDO);
		HashMap<String, String> data = new HashMap<String, String>();
		data.put("url", "main.htm");
		result.setStatus(1);
		result.setData(data);
		showJson(result, writer);
	}

	@RequestMapping(value = "doedit.htm", method = RequestMethod.POST)
	public void doEdit(Writer writer, @Valid AddressBookDO AddressBookDO,
			BindingResult BindingResult) {
		ReturnJsonDO result = new ReturnJsonDO();
		if (BindingResult.hasErrors()) {
			List<ObjectError> errors = BindingResult.getAllErrors();
			String msgString = "";
			for (ObjectError value : errors) {
				msgString += value.getDefaultMessage() + "\r\n";
			}
			result.setStatus(0);
			result.setMsg(msgString);
			showJson(result, writer);
			return;
		}
		addressBookService.update(AddressBookDO);
		HashMap<String, String> data = new HashMap<String, String>();
		data.put("url", "main.htm");
		result.setStatus(1);
		result.setData(data);
		showJson(result, writer);
	}

	@RequestMapping("logout.htm")
	public void logout(HttpServletResponse response) {
		request.getSession().removeAttribute("islogin");
		request.getSession().removeAttribute("user");
		try {
			response.sendRedirect("login.htm");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@RequestMapping(value = "del.htm", method = RequestMethod.POST)
	public void del(Writer writer) {
		String idStr = request.getParameter("id");
		int id = 0;
		int uid = ((UserDO) request.getSession().getAttribute("user")).getUid();
		ReturnJsonDO result = new ReturnJsonDO();
		if (idStr != null && idStr != "") {
			try {
				id = Integer.parseInt(idStr);
			} catch (Exception e) {
			}
		}
		if (id < 0) {
			result.setStatus(0);
			result.setMsg("ID非法");
			showJson(result, writer);
			return;
		}
		AddressBookDO book = addressBookService.getById(id);
		if (book != null && book.getUid() != uid) {
			result.setStatus(0);
			result.setMsg("无权限删除");
			showJson(result, writer);
			return;
		}
		addressBookService.delete(id);
		result.setStatus(1);
		showJson(result, writer);
	}

	@RequestMapping(value = "import.htm", method = RequestMethod.GET)
	public String join() {
		return "import";
	}

	@RequestMapping("error.htm")
	public String Error(Model model) {
		String msg = request.getParameter("msg");
		model.addAttribute("msg", msg);
		return "error";
	}

	@RequestMapping(value = "doimport.htm", method = RequestMethod.POST)
	public void doJoin(HttpServletResponse response,
			@RequestParam("img") MultipartFile file) throws IOException {
		if (file.isEmpty()) {
			response.sendRedirect("error.htm?msg=请选择要上传的文件");
			return;
		}
		String filename = file.getOriginalFilename();
		if (!this.getExtensionName(filename).equals("vcf")) {
			response.sendRedirect("error.htm?msg=只允许上传vcf格式的vcard文件");
			return;
		}
		String fileString = this.copyFile(file.getInputStream(),
				file.getOriginalFilename());

		File bigVCardFile = new File("src/main/webapp/resources/upload/"
				+ fileString);
		VCardEngine vcardEngine = new VCardEngine();
		List<VCard> vcardsList = vcardEngine.parseMultiple(bigVCardFile);
		for (VCard value : vcardsList) {
			AddressBookDO book = vcardService.getBook(value);
			book.setUid(((UserDO) request.getSession().getAttribute("user"))
					.getUid());
			addressBookService.insert(book);
		}
		response.sendRedirect("main.htm");
	}

	@RequestMapping(value = "export.htm", method = RequestMethod.GET)
	public void export(Writer w, HttpServletResponse response) {
		int uid = ((UserDO) request.getSession().getAttribute("user")).getUid();
		int total = addressBookService.getByUidCount(uid);
		List<AddressBookDO> data = addressBookService.getList(uid, 0, total);
		String vString = "";
		for (AddressBookDO value : data) {
			VCardImpl vCardImpl = vcardService.getVcard(value);
			VCardWriter writer = new VCardWriter();
			writer.setOutputVersion(VCardVersion.V3_0);
			writer.setCompatibilityMode(CompatibilityMode.RFC2426);
			writer.setFoldingScheme(FoldingScheme.MIME_DIR);
			writer.setBinaryfoldingScheme(BinaryFoldingScheme.MIME_DIR);
			writer.setEOL(VCardUtils.LF);
			writer.setVCard(vCardImpl);
			vString += writer.buildVCardString();
		}

		try {
			response.setHeader("Charset", "utf8");
			response.setHeader("Content-Type", "text/vcard");
			response.setHeader("Content-Disposition",
					"attachment;filename=vcard.vcf");
			w.write(vString);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void showJson(ReturnJsonDO result, Writer writer) {
		Gson gson = new Gson();
		try {
			writer.write(gson.toJson(result));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 写文件到本地
	 * 
	 * @param in
	 * @param fileName
	 * @throws IOException
	 */
	@SuppressWarnings("unused")
	private String copyFile(InputStream in, String fileName) throws IOException {
		String extName = this.getExtensionName(fileName);
		long currentTime = new Date().getTime();
		String filePath = DigestUtils.md5DigestAsHex(java.lang.Long.toString(
				currentTime).getBytes())
				+ "." + extName;
		FileOutputStream fs = new FileOutputStream(
				"src/main/webapp/resources/upload/" + filePath);
		byte[] buffer = new byte[1024 * 1024];
		int bytesum = 0;
		int byteread = 0;
		while ((byteread = in.read(buffer)) != -1) {
			bytesum += byteread;
			fs.write(buffer, 0, byteread);
			fs.flush();
		}
		fs.close();
		in.close();
		return filePath;
	}

	private String getExtensionName(String filename) {
		if ((filename != null) && (filename.length() > 0)) {
			int dot = filename.lastIndexOf('.');
			if ((dot > -1) && (dot < (filename.length() - 1))) {
				return filename.substring(dot + 1);
			}
		}
		return filename;
	}
}
